import 'package:flutter/cupertino.dart';
import 'package:zhiying_base_widget/widgets/home/home_goods/models/home_goods_model.dart';
import 'package:zhiying_comm/zhiying_comm.dart';

class SearchResultGoodsListRepository {
  SearchResultGoodsListRepository({@required this.reqData});

  /// 当前页面
  int _currentPage = 1;

  /// 最大条数
  final int MAX = 10;

  /// 是否还有更多数据
  bool _hasNomore = true;

  /// 数据
  List<HomeGoodsModel> _oldDatas = [];

  /// 显示样式
  bool _isShowOneColumn = true;

  /// 查询数据参数
  Map<String, dynamic> reqData = {};

  /// 初始化
  Future<List<HomeGoodsModel>> fetchInitData() async {
    _currentPage = 1;
    _hasNomore = true;
    return _baseInitData(true, reqData);
  }

  /// 下拉刷新
  Future<List<HomeGoodsModel>> fetchRefreshData() async {
    _currentPage = 1;
    _hasNomore = true;
    return _baseInitData(true,reqData);
  }

  /// 上拉更多
  Future<List<HomeGoodsModel>> fetchLoadData() async {
    /// 只有更多数据的时候才进行加载
    if (_hasNomore) {
      return _baseInitData(false, reqData);
    } else {
      return null;
    }
  }

  /// 修改显示样式
  Map<String, dynamic> fetchChangeStyle() {
    _isShowOneColumn = !_isShowOneColumn;
    return {'style': _isShowOneColumn, 'datas': _oldDatas};
  }

  /// 输入框修改关键字
  Future<List<HomeGoodsModel>> fetchSearchSubmit(String keywords) async{
    if(!EmptyUtil.isEmpty(keywords)) {
      reqData['reqData'] = keywords;
      _currentPage = 1;
      _hasNomore =true;
      return _baseInitData(true, reqData);
    }
    return null;
  }

  /// 查询
  Future<List<HomeGoodsModel>> _baseInitData(bool refresh,final Map<String, dynamic> model) async {
    String keyword = model['keywords'] ?? '';
    String type = model['type'];
    if (EmptyUtil.isEmpty(keyword)) return null;
    var result = await NetUtil.post('/api/v1/s/$type',
        params: {
          'keyword': keyword,
          'size': MAX.toString(),
          'p': _currentPage.toString(),
        },
        method: NetMethod.POST);
    try {
      // search_list
      if (NetUtil.isSuccess(result) && !EmptyUtil.isEmpty(result[GlobalConfig.HTTP_RESPONSE_KEY_DATA])) {
        List data = List.from(result[GlobalConfig.HTTP_RESPONSE_KEY_DATA]['search_list']);
        if (data.length >= MAX) {
          _hasNomore = true;
          ++_currentPage;
        }
        // 下拉刷新需要清理
        if(refresh){
          _oldDatas.clear();
        }

        data.forEach((item) {
          _oldDatas.add(HomeGoodsModel.fromJson(item));
        });

        List<HomeGoodsModel> newData = [];
        newData.addAll(_oldDatas);

        return newData;
      }
    } catch (e) {
      Logger.error(e);
    }
    _hasNomore = false;
    return null;
  }
}
